
import UIKit

class MyCollectionViewController: UICollectionViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      var config = UICollectionLayoutListConfiguration(appearance: .grouped)
      config.headerMode = .supplementary
      let layout = UICollectionViewCompositionalLayout.list(using: config)
      collectionView.collectionViewLayout = layout

      prepareDataSource()
      prepareSnapshot()
   }
   func prepareDataSource() {
      let cellRegistration = UICollectionView.CellRegistration<UICollectionViewListCell, ItemsData.ID> { cell, indexPath, itemID in
         if let item = AppData.items.first(where: { $0.id == itemID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item.name
            config.secondaryText = "Calories: \(item.calories)"
            config.image = UIImage(named: item.image)
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config
         }
      }
      AppData.dataSource = UICollectionViewDiffableDataSource<Sections.ID, ItemsData.ID>(collectionView: collectionView) { (collection, indexPath, itemID) in
         return collection.dequeueConfiguredReusableCell(using: cellRegistration, for: indexPath, item: itemID)
      }
      let headerRegistration = UICollectionView.SupplementaryRegistration<MyHeader>(elementKind: UICollectionView.elementKindSectionHeader) { headerView, kind, indexPath in
         headerView.textView.text = AppData.sections[indexPath.section].name
      }
      AppData.dataSource.supplementaryViewProvider = { view, kind, indexPath in
         return self.collectionView.dequeueConfiguredReusableSupplementary(using: headerRegistration, for: indexPath)
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections.ID, ItemsData.ID>()
      snapshot.appendSections(AppData.sections.map({ $0.id }))
      for section in AppData.sections {
         let itemIDs = AppData.items.compactMap({ value in
            return value.section == section.name ? value.id : nil
         })
         snapshot.appendItems(itemIDs, toSection: section.id)
      }
      AppData.dataSource.apply(snapshot)
   }
}

