
import UIKit

class MyCollectionViewController: UICollectionViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      var config = UICollectionLayoutListConfiguration(appearance: .grouped)
      config.headerMode = .supplementary
      config.separatorConfiguration.color = .systemRed

      config.itemSeparatorHandler = { indexPath, config in
         let row = indexPath.item
         let section = indexPath.section

         var lastRow = 0
         if let sectionID = AppData.dataSource.sectionIdentifier(for: section) {
            lastRow = AppData.dataSource.snapshot().numberOfItems(inSection: sectionID)
            lastRow = lastRow > 0 ? lastRow - 1 : 0
         }
         var configuration = config
         configuration.topSeparatorVisibility = row == 0 ? .hidden : .automatic
         configuration.bottomSeparatorVisibility = row == lastRow ? .hidden : .automatic
         return configuration
      }
      let layout = UICollectionViewCompositionalLayout.list(using: config)
      collectionView.collectionViewLayout = layout

      prepareDataSource()
      prepareSnapshot()
   }
   func prepareDataSource() {
      let cellRegistration = UICollectionView.CellRegistration<UICollectionViewListCell, ItemsData.ID> { cell, indexPath, itemID in
         if let item = AppData.items.first(where: { $0.id == itemID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item.name
            config.secondaryText = "Calories: \(item.calories)"
            config.image = UIImage(named: item.image)
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config

            let selected = item.selected
            cell.accessories = selected ? [.checkmark()] : []
         }
      }
      AppData.dataSource = UICollectionViewDiffableDataSource<Sections.ID, ItemsData.ID>(collectionView: collectionView) { (collection, indexPath, itemID) in
         return collection.dequeueConfiguredReusableCell(using: cellRegistration, for: indexPath, item: itemID)
      }
      let headerRegistration = UICollectionView.SupplementaryRegistration<MyHeader>(elementKind: UICollectionView.elementKindSectionHeader) { headerView, kind, indexPath in
         headerView.textView.text = AppData.sections[indexPath.section].name
      }
      AppData.dataSource.supplementaryViewProvider = { view, kind, indexPath in
         return self.collectionView.dequeueConfiguredReusableSupplementary(using: headerRegistration, for: indexPath)
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections.ID, ItemsData.ID>()
      snapshot.appendSections(AppData.sections.map({ $0.id }))
      for section in AppData.sections {
         let itemIDs = AppData.items.compactMap({ value in
            return value.section == section.name ? value.id : nil
         })
         snapshot.appendItems(itemIDs, toSection: section.id)
      }
      AppData.dataSource.apply(snapshot)
   }
   override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
      if let itemID = AppData.dataSource.itemIdentifier(for: indexPath) {
         if let item = AppData.items.first(where: { $0.id == itemID }) {
            item.selected.toggle()

            var current = AppData.dataSource.snapshot()
            current.reconfigureItems([itemID])
            AppData.dataSource.apply(current)

            collectionView.deselectItem(at: indexPath, animated: true)
         }
      }
   }
}

