
import UIKit

class MyCollectionViewController: UICollectionViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      var config = UICollectionLayoutListConfiguration(appearance: .grouped)
      config.headerMode = .supplementary

      config.trailingSwipeActionsConfigurationProvider = { indexPath in
         let button = UIContextualAction(style: .normal, title: "Remove", handler: { (action, view, completion) in
            if let itemID = AppData.dataSource.itemIdentifier(for: indexPath), let sectionID = AppData.dataSource.sectionIdentifier(for: indexPath.section) {
               AppData.items.removeAll(where: { $0.id == itemID })

               var currentSnapshot = AppData.dataSource.snapshot()
               currentSnapshot.deleteItems([itemID])
               if currentSnapshot.numberOfItems(inSection: sectionID) <= 0 {
                  AppData.sections.removeAll(where: { $0.id == sectionID })
                  currentSnapshot.deleteSections([sectionID])
               }
               AppData.dataSource.apply(currentSnapshot)
            }
            completion(true)
         })
         let config = UISwipeActionsConfiguration(actions: [button])
         return config
      }
      let layout = UICollectionViewCompositionalLayout.list(using: config)
      collectionView.collectionViewLayout = layout

      prepareDataSource()
      prepareSnapshot()
   }
   func prepareDataSource() {
      let cellRegistration = UICollectionView.CellRegistration<UICollectionViewListCell, ItemsData.ID> { cell, indexPath, itemID in
         if let item = AppData.items.first(where: { $0.id == itemID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item.name
            config.secondaryText = "Calories: \(item.calories)"
            config.image = UIImage(named: item.image)
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config

            let selected = item.selected
            cell.accessories = selected ? [.checkmark()] : []
         }
      }
      AppData.dataSource = UICollectionViewDiffableDataSource<Sections.ID, ItemsData.ID>(collectionView: collectionView) { (collection, indexPath, itemID) in
         return collection.dequeueConfiguredReusableCell(using: cellRegistration, for: indexPath, item: itemID)
      }
      let headerRegistration = UICollectionView.SupplementaryRegistration<MyHeader>(elementKind: UICollectionView.elementKindSectionHeader) { headerView, kind, indexPath in
         headerView.textView.text = AppData.sections[indexPath.section].name
      }
      AppData.dataSource.supplementaryViewProvider = { view, kind, indexPath in
         return self.collectionView.dequeueConfiguredReusableSupplementary(using: headerRegistration, for: indexPath)
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections.ID, ItemsData.ID>()
      snapshot.appendSections(AppData.sections.map({ $0.id }))
      for section in AppData.sections {
         let itemIDs = AppData.items.compactMap({ value in
            return value.section == section.name ? value.id : nil
         })
         snapshot.appendItems(itemIDs, toSection: section.id)
      }
      AppData.dataSource.apply(snapshot)
   }
   override func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
      if let itemID = AppData.dataSource.itemIdentifier(for: indexPath) {
         if let item = AppData.items.first(where: { $0.id == itemID }) {
            item.selected.toggle()

            var current = AppData.dataSource.snapshot()
            current.reconfigureItems([itemID])
            AppData.dataSource.apply(current)

            collectionView.deselectItem(at: indexPath, animated: true)
         }
      }
   }
}

