
import UIKit

class MyCollectionViewController: UICollectionViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      let config = UICollectionLayoutListConfiguration(appearance: .plain)
      let layout = UICollectionViewCompositionalLayout.list(using: config)
      collectionView.collectionViewLayout = layout

      prepareDataSource()
      prepareSnapshot()
   }
   func prepareDataSource() {
      let cellRegistration = UICollectionView.CellRegistration<UICollectionViewListCell, MainItems.ID> { cell, indexPath, itemID in
         if let item = self.getItem(id: itemID) {
            var config = cell.defaultContentConfiguration()
            config.text = item.name
            cell.contentConfiguration = config
            cell.accessories = item.options != nil ? [.outlineDisclosure()] : []
         }
      }
      AppData.dataSource = UICollectionViewDiffableDataSource<Sections, MainItems.ID>(collectionView: collectionView) { (collection, indexPath, itemID) in
         return collection.dequeueConfiguredReusableCell(using: cellRegistration, for: indexPath, item: itemID)
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSectionSnapshot<MainItems.ID>()
      for mainItem in AppData.items {
         snapshot.append([mainItem.id], to: nil)
         snapshot.append(mainItem.options.map({ $0.id }), to: mainItem.id)
      }
      AppData.dataSource.apply(snapshot, to: .main, animatingDifferences: false)
   }
   func getItem(id: MainItems.ID) -> MainItems? {
      var item = AppData.items.first(where: { $0.id == id })
      if item == nil {
         for main in AppData.items {
            if let found = main.options.first(where: { $0.id == id }) {
               item = found
               break
            }
         }
      }
      return item
   }
}

