
import UIKit

class MyCollectionViewController: UICollectionViewController {
   override func viewDidLoad() {
      super.viewDidLoad()
      let customLayout = createLayout()
      collectionView.collectionViewLayout = customLayout
      prepareDataSource()
      prepareSnapshot()
   }
   func prepareDataSource() {
      let cellRegistration = UICollectionView.CellRegistration<FoodCell, ItemsData.ID> { cell, indexPath, itemID in
         if let item = AppData.items.first(where: { $0.id == itemID }) {
            cell.picture.image = UIImage(named: item.image)
         }
      }
      AppData.dataSource = UICollectionViewDiffableDataSource<Sections, ItemsData.ID>(collectionView: collectionView) { (collection, indexPath, itemID) in
         return collection.dequeueConfiguredReusableCell(using: cellRegistration, for: indexPath, item: itemID)
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections, ItemsData.ID>()
      snapshot.appendSections([.main])
      snapshot.appendItems(AppData.items.map({ $0.id }))
      AppData.dataSource.apply(snapshot)
   }
   func createLayout() -> UICollectionViewLayout {
      let itemSize = NSCollectionLayoutSize(widthDimension: .fractionalWidth(0.2), heightDimension: .fractionalWidth(0.2))
      let item = NSCollectionLayoutItem(layoutSize: itemSize)
      item.contentInsets = NSDirectionalEdgeInsets(top: 10, leading: 5, bottom: 5, trailing: 10)

      let groupSize = NSCollectionLayoutSize(widthDimension: .fractionalWidth(1), heightDimension: .fractionalWidth(0.2))
      let group = NSCollectionLayoutGroup.horizontal(layoutSize: groupSize, subitems: [item])

      let section = NSCollectionLayoutSection(group: group)

      let layout = UICollectionViewCompositionalLayout(section: section)
      return layout
   }
}

